/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is ForecastFox.
 *
 * The Initial Developer of the Original Code is
 * Jon Stritar <jstritar@MIT.EDU>.
 * Portions created by the Initial Developer are Copyright (C) 2005
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 * Jon Stritar <jstritar@MIT.EDU>
 * Richard Klein <richwklein@mchsi.com>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

var gLabels = null;

function labelsLoad()
{
  gLabels = new LabelsModule();
  gLabels.init();
}

function labelsUnload()
{
  gLabels.destroy();
  gLabels = null;
}

function LabelsModule()
{
  this.wrappedJSObject = this;
  return this;
}

LabelsModule.prototype = {
  mUpdate: null,
  mProfiles: null,
  mBranch: null,
  mBundle: null,  
  mType: null,
  mDefaults: null,
  mClipboard: null,
  mSupport: null,
  mTitle: null,
  mLabel: null,
  mTitleBtn: null,
  mLabelBtn: null,
  mData: null,
  
  init: function()
  {
    //setup variables
    this.mUpdate = Components.classes["@mozdev.org/forecastfox/update;1"].getService(Components.interfaces.nsIForecastfoxUpdate);
    this.mProfiles = Components.classes["@mozdev.org/forecastfox/profiles;1"].getService(Components.interfaces.nsIForecastfoxProfiles);
    var prefService = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService); 
    this.mBundle = document.getElementById("ff-bundle-labels");
    this.mTitle = document.getElementById("ff-text-title");
    this.mLabel = document.getElementById("ff-text-label");
    this.mTitleBtn = document.getElementById("ff-btn-title");
    this.mLabelBtn = document.getElementById("ff-btn-label");    
    this.mBranch = prefService.getBranch("forecastfox.");
    this.mClipboard = Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
    this.mSupport = Components.classes["@mozilla.org/supports-string;1"].createInstance(Components.interfaces.nsISupportsString);

    this.mData = new Array();
    this.mData[0] = [["/n", this.mBundle.getString("ff.labels.nl")]];
    this.mData[1] = [["[tm]", this.mBundle.getString("ff.labels.tm")],
                     ["[prof]", this.mBundle.getString("ff.labels.prof")],
                     ["[dnam]", this.mBundle.getString("ff.labels.dnam")],
                     ["[lat]", this.mBundle.getString("ff.labels.lat")],
                     ["[lon]", this.mBundle.getString("ff.labels.lon")],
                     ["[zone]", this.mBundle.getString("ff.labels.zone")],
                     ["[icon]", this.mBundle.getString("ff.labels.icon")],
                     ["[sunr]", this.mBundle.getString("ff.labels.sunr")],
                     ["[suns]", this.mBundle.getString("ff.labels.suns")],
                     ["[t]", this.mBundle.getString("ff.labels.t")],
                     ["[winds]", this.mBundle.getString("ff.labels.winds")],
                     ["[windgust]", this.mBundle.getString("ff.labels.windgust")],
                     ["[windd]", this.mBundle.getString("ff.labels.windd")],
                     ["[windt]", this.mBundle.getString("ff.labels.windt")]]; 
                        
    this.mData[2] = [["[lsup]", this.mBundle.getString("ff.labels.lsup")],
                     ["[obst]", this.mBundle.getString("ff.labels.obst")],
                     ["[tmp]", this.mBundle.getString("ff.labels.tmp")],
                     ["[flik]", this.mBundle.getString("ff.labels.flik")],
                     ["[barr]", this.mBundle.getString("ff.labels.barr")],
                     ["[bard]", this.mBundle.getString("ff.labels.bard")],
                     ["[hmid]", this.mBundle.getString("ff.labels.hmid")],
                     ["[vis]", this.mBundle.getString("ff.labels.vis")],
                     ["[uvi]", this.mBundle.getString("ff.labels.uvi")],
                     ["[uvt]", this.mBundle.getString("ff.labels.uvt")],
                     ["[dewp]", this.mBundle.getString("ff.labels.dewp")],
                     ["[moont]", this.mBundle.getString("ff.labels.moont")]];

    this.mData[3] = [["[part]", this.mBundle.getString("ff.labels.part")],
                     ["[day]", this.mBundle.getString("ff.labels.day")],
                     ["[sday]", this.mBundle.getString("ff.labels.sday")],
                     ["[date]", this.mBundle.getString("ff.labels.date")],
                     ["[lsup]", this.mBundle.getString("ff.labels.lsup")],
                     ["[tmp]", this.mBundle.getString("ff.labels.tmp2")],
                     ["[hi]", this.mBundle.getString("ff.labels.hi")],
                     ["[low]", this.mBundle.getString("ff.labels.low")],
                     ["[ppcp]", this.mBundle.getString("ff.labels.ppcp")],
                     ["[hmid]", this.mBundle.getString("ff.labels.hmid")]];
                                       
    //get type
    this.mType = window.arguments[0];
    
    //get defaults
    var defaultBranch = prefService.getDefaultBranch("forecastfox.");
    this.mDefaults = {};
    this.mDefaults[this.mType + ".label"] = this.mBundle.getString(defaultBranch.getCharPref(this.mType + ".label"));
    if (this.mType == "cc.slider")
      this.mDefaults[this.mType + ".title"] = this.mBundle.getString(defaultBranch.getCharPref(this.mType + ".title"));  
  
    //set display
    document.getElementById("ff-desc-type").value = this.mBundle.getString("ff.labels." + this.mType);
    if (this.mType == "cc.slider") {
      document.getElementById("ff-box-title").removeAttribute("hidden");
      if (this.mBranch.prefHasUserValue("cc.slider.title")) {
        this.mTitle.value = this.getUTFOption("cc.slider.title");
        this.mTitleBtn.removeAttribute("disabled");
      } else {
        this.mTitle.value = this.mDefaults["cc.slider.title"];
        this.mTitleBtn.setAttribute("disabled", "true");      
      }
      this.mTitle.focus();
      this.mTitle.select();
      
    } else {
      this.mLabel.focus();
      this.mLabel.select();    
    }
    if (this.mBranch.prefHasUserValue(eval("'" + this.mType + ".label'"))) {
      this.mLabel.value = this.getUTFOption(eval("'" + this.mType + ".label'"));
      this.mLabelBtn.removeAttribute("disabled");
    } else {
      this.mLabel.value = this.mDefaults[eval("'" + this.mType + ".label'")];
      this.mLabelBtn.setAttribute("disabled", "true");
    }
    
    this.populateTree();
    this.updateButtons();
   
    // disable the apply button
    document.getElementById("apply").setAttribute("disabled", true);
  },
  
  destroy: function()
  {
    this.mUpdate = null;
    this.mProfiles = null;
    this.mBranch = null;
    this.mType = null;
    this.mDefaults = null;
    this.mClipboard = null;
    this.mSupport = null;
    this.mTitle = null;
    this.mLabel = null;
    this.mTitleBtn = null;
    this.mLabelBtn = null;
    this.mData = null;
  },
  
  getUTFOption: function(aOption)
  {
    return this.mBranch.getComplexValue(aOption,Components.interfaces.nsISupportsString).data;  
  },
  
  setUTFOption: function(aOption, aValue)
  {
    this.mSupport.data = aValue;
    this.mBranch.setComplexValue(aOption,Components.interfaces.nsISupportsString, this.mSupport);  
  },
  
  updateButtons: function(aEvent)
  {
    // don't enable the apply button for button presses
    if (aEvent) { 
      if ((aEvent.originalTarget.localName == "button") || 
        (aEvent.originalTarget.localName == "menuitem"))
      return;
    }
    document.getElementById("apply").removeAttribute("disabled");
    
    //set restore buttons
    if (this.mTitle.value != this.mDefaults[eval("'" + this.mType + ".title'")])
      this.mTitleBtn.removeAttribute("disabled");
    else
      this.mTitleBtn.setAttribute("disabled", "true");
    if (this.mLabel.value != this.mDefaults[eval("'" + this.mType + ".label'")])
      this.mLabelBtn.removeAttribute("disabled");
    else
      this.mLabelBtn.setAttribute("disabled", "true");        
  },
  
  copy: function(aType)
  {
    var tree = document.getElementById("ff-tree-vars");  
    if (tree.currentIndex == -1)
      return;

    var item = tree.contentView.getItemAtIndex(tree.currentIndex);
    var cells = item.getElementsByTagName("treecell");
    var copy_string;

    switch (aType) {
      case "variable" :
        copy_string = cells[0].getAttribute("label");
        break;
      case "description" :
        copy_string = cells[1].getAttribute("label");
        break;
    }

    this.mClipboard.copyString(copy_string);  
  },
  
  populateTree: function()
  {   
    var tree = document.getElementById("ff-tree-items");
    var x;
    
    //clear old tree data
    while (tree.hasChildNodes())
      tree.removeChild(tree.lastChild);
    
    //set /n variable
    if (this.mType.match("tooltip") != null)
      this.addTreeRow(tree, this.mData[0][0]);
    
    //set common variables
    for (x=0; x<this.mData[1].length; x++)
      this.addTreeRow(tree, this.mData[1][x]);
    
    //set specific variables
    var type = this.mType.split(".");
    var index = (type[0] == "cc") ? 2 : 3;
    for (x=0; x<this.mData[index].length; x++)
      this.addTreeRow(tree, this.mData[index][x]);     
  },

  addTreeRow: function(aParent, aData) 
  {
    var item = document.createElement("treeitem");
    var node = document.createElement("treerow");
    var cell;

    for (var value in aData) {
      // Create the cell
      cell = document.createElement("treecell");
      cell.setAttribute("label", aData[value]);

      // And add it onto the row.
      node.appendChild(cell);
    }

    // When done add the node onto the item, then the item onto the aParent.
    item.appendChild(node);
    aParent.appendChild(item);
  },
    
  restoreDefault: function(aType)
  {
    if (aType == "title")
      this.mTitle.value = this.mDefaults[eval("'" + this.mType + ".title'")];
    else
      this.mLabel.value = this.mDefaults[eval("'" + this.mType + ".label'")];    

    this.updateButtons();  
  },
  
  validate: function()
  {
    //XXX Needs done
    return true;
  },
  
  accept: function(aClose)
  {
    if (!this.validate())
      return;

    //set values
    if (this.mType == "cc.slider") {
      if ((this.mTitle.value == this.mDefaults[eval("'" + this.mType + ".title'")]) &&
          (this.mBranch.prefHasUserValue(eval("'" + this.mType + ".title'"))))
        this.mBranch.clearUserPref(eval("'" + this.mType + ".title'"));
      else
        this.setUTFOption(eval("'" + this.mType + ".title'"), this.mTitle.value);
    
    } 
      
    if ((this.mLabel.value == this.mDefaults[eval("'" + this.mType + ".label'")]) &&
        (this.mBranch.prefHasUserValue(eval("'" + this.mType + ".label'"))))
      this.mBranch.clearUserPref(eval("'" + this.mType + ".label'"));
    else
      this.setUTFOption(eval("'" + this.mType + ".label'"), this.mLabel.value);
    
    //close
    if (aClose)  
      window.close();
    else
      document.getElementById("apply").setAttribute("disabled", "true");  
  },
    
  openLink: function(aEvent, aTWC)
  {
    var url = aEvent.currentTarget.getAttribute("url");
    if (aTWC)
      url = url + "?prod=" + this.mUpdate.productCode + "&par=" + this.mUpdate.partnerId;

    window.openDialog("chrome://browser/content/browser.xul", "_blank", "chrome,all,dialog=no", url, null, null);
  }  
}
